#ifndef OPEN1X_DAEMON_H
#define OPEN1X_DAEMON_H

// There are two different ways that the supplicant can communicate with a
// client program.  One is for the client program to request information,
// the other is for the supplicant to push information to the client program.

// The format for communication in either direction is :
//  xx interface value

// Where xx is a hex value as defined below, interface is the name of the
// interface that the event happened on, and "value" is some other information
// that is needed to make the event meaningful.

// Messages sent from the client program, to the supplicant should be in the
// range of 0-99, while messages from the supplicant to the client should be
// in the range of 100-199.  Error messages, EAP Alerts, or other strange
// messages should be in the range of 200-255.

// Messages from the client program to the supplicant.
#define SET_VALUE    1  /* Assign a value to a variable. */
#define GET_STATE    2  /* Get the current state of the state machine. */
#define CLIENT_ACK   3  /* ACK the Supplicant. (Usually in response to a TEST_CLIENT message.) */
#define GET_VALUE    4  /* Get a value of a variable. */
#define GET_INTS     5  /* List the interfaces the Supplicant is working with. */
#define GET_HI_STATE 6  /* Return the highest state of all of the interfaces the Supplicant is working with. */
#define REGISTER_GUI 7  /* Request that the supplicant remember us, and send us push messages. */

// Messages from the supplicant to the client program
#define PUSH_ERR    101 /* Send an error message to the client. */
#define TEST_CLIENT 102 /* Check to see if a client is there. */
#define STATE_RESP  103 /* Answer a GET_STATE request. */
#define GET_RESP    104 /* Answer a GET_VALUE request. */
#define SET_RESP    105 /* Answer a SET_VALUE request. */
#define INTS_RESP   106 /* Answer a GET_INTS request. */
#define GET_HI_RESP 107 /* Answer a GET_HI_STATE request. */
#define PUSH_GET_USER_PW 108  /* Request that the GUI give us a username/pwd. */
#define GUI_REGISTERED 109 /* Ack the GUI registration. */

// Other messages
#define EAP_MESSAGE 200  /* Send an EAP-Notify */

/* Store known clients.  Used to push messages to clients. */
struct client_sockets {
  struct sockaddr *addr;
  int addr_size;
  int failed_queries;
  struct client_sockets *next;
};


int serverSockInit();
int handleMessages();
void sendMessage(char, char *, char *, struct sockaddr *, int);
void lower_str(char *);

extern int msgSock;
extern int serverSock;

#endif
