/**
 * A client-side 802.1x implementation supporting EAP/SIM
 *
 * This code is released under both the GPL version 2 and BSD licenses.
 * Either license may be used.  The respective licenses are found below.
 *
 * Copyright (C) 2003 Chris Hessing
 * All Rights Reserved
 *
 * --- GPL Version 2 License ---
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * --- BSD License ---
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the University of
 *       Maryland at College Park and its contributors.
 *  - Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************
* EAPOL Function implementations for supplicant
 * 
 * File: sm_handler.c
 *
 * Authors: Chris.Hessing@utah.edu
 *
 * $Id: sm_handler.c,v 1.5 2003/09/25 00:10:41 chessing Exp $
 * $Date: 2003/09/25 00:10:41 $
 * $Log: sm_handler.c,v $
 * Revision 1.5  2003/09/25 00:10:41  chessing
 * Clean up some memory leaks.
 *
 * Revision 1.4  2003/09/24 20:17:04  chessing
 * EAP-SIM code tagged to show that it was funded by Internet Foundation Austria.
 *
 * Revision 1.3  2003/09/23 04:22:54  chessing
 * Code cleanups.
 *
 * Revision 1.2  2003/09/21 03:59:35  chessing
 * Various code cleanups, and other updates.  Smartcard functions for EAP-SIM are more stable.
 *
 * Revision 1.1  2003/09/18 23:03:38  chessing
 * Memory leak fixes, inital implementation of EAP-SIM (draft 5 only).  EAP-SIM still has a lot of bugs to be fixed, and doesn't key correctly yet.  EAP-SIM also requires libpcsc, and pthread.  To enable EAP-SIM use the --enable-eap-sim flag on configure.
 *
 *******************************************************************/

/*******************************************************************
 *
 * The development of the EAP/SIM support was funded by Internet
 * Foundation Austria (http://www.nic.at/ipa)
 *
 *******************************************************************/


/* Interface to Smart Cards using PCSC with 802.1x.  */


/* Taken from code by Michael Haberler    mah@eunet.at */
/* which was based on work by marek@bmlv.gv.at */

#ifdef EAP_SIM_ENABLE

#include <stdio.h>
#include <winscard.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "../mschapv2/eapmschapv2.h"  // Needed for ctonibble function.

int need_init = 1;   // By default, we need an init to start with.

#ifndef SCDEBUG
#define SCDEBUG  1
#endif

#define MAXBUFF  (512)

#define SELECT_MF       "A0A40000023F00"
#define SELECT_DF_GSM   "A0A40000027F20"
#define SELECT_EF_IMSI  "A0A40000026F07"
#define RUN_GSM         "A088000010"
#define GET_IMSI        "A0B0000009"

char *o_readername;
SCARDCONTEXT g_sc_context;
SCARDHANDLE g_card_hdl = 0;
SCARD_IO_REQUEST scir;

unsigned long o_stdprotocol;
DWORD readerstrlen;

int debug;


void print_sc_error(long err)
{
  switch (err)
    {
    case SCARD_S_SUCCESS:
      printf("Successful card call!\n");
      break;
    case SCARD_E_CANCELLED:
      printf("Error : Card Request Cancelled!\n");
      break;
    case SCARD_E_CANT_DISPOSE:
      printf("Error : Can't dispose (!?)\n");
      break;
    case SCARD_E_INSUFFICIENT_BUFFER:
      printf("Error : Insufficient Buffer\n");
      break;
    case SCARD_E_INVALID_ATR:
      printf("Error : Invalid ATR\n");
      break;
    case SCARD_E_INVALID_HANDLE:
      printf("Error : Invalid handle\n");
      break;
    case SCARD_E_INVALID_PARAMETER:
      printf("Error : Invalid parameter\n");
      break;
    case SCARD_E_INVALID_TARGET:
      printf("Error : Invalid target\n");
      break;
    case SCARD_E_INVALID_VALUE:
      printf("Error : Invalid Value\n");
      break;
    case SCARD_E_NO_MEMORY:
      printf("Error : No memory\n");
      break;
    case SCARD_F_COMM_ERROR:
      printf("Error : Communication error \n");
      break;
    case SCARD_F_INTERNAL_ERROR:
      printf("Error : Internal error\n");
      break;
    case SCARD_F_WAITED_TOO_LONG:
      printf("Error : Waited too long\n");
      break;
    case SCARD_E_UNKNOWN_READER:
      printf("Error : Unknown reader\n");
      break;
    case SCARD_E_TIMEOUT:
      printf("Error : Timeout\n");
      break;
    case SCARD_E_SHARING_VIOLATION:
      printf("Error : Sharing Violation\n");
      break;
    case SCARD_E_NO_SMARTCARD:
      printf("Error : No smartcard!\n");
      break;
    case SCARD_E_UNKNOWN_CARD:
      printf("Error : Unknown card!\n");
      break;
    case SCARD_E_PROTO_MISMATCH:
      printf("Error : Protocol mismatch!\n");
      break;
    case SCARD_E_NOT_READY:
      printf("Error : Not ready!\n");
      break;
    case SCARD_E_SYSTEM_CANCELLED:
      printf("Error : System Cancelled\n");
      break;
    case SCARD_E_NOT_TRANSACTED:
      printf("Error : Not Transacted\n");
      break;
    case SCARD_E_READER_UNAVAILABLE:
      printf("Error : Reader unavailable\n");
      break;
    case SCARD_F_UNKNOWN_ERROR:
    default:
      printf("Unknown error!\n");
      break;
    }
}

void strtohex(char *instr, char *outstr, int *blen)
{
  int i;
  char val1,val2;

  for (i=0;i<(strlen(instr)/2);i++)
    {
      val1=ctonibble(instr[i*2]);
      val2=ctonibble(instr[(i*2)+1]);
      outstr[i]=((val1<<4)+val2);
    }
  *blen = (strlen(instr)/2);
}

  

int card_io(char *cmd, LPBYTE outbuff, LPDWORD olen)
{
  static char g_getresponse[5]= {0xa0,0xc0,0x00,0x00 };
  int cmdlen, ret;
  char *bcmd;

  cmdlen = strlen(cmd)/2;
  bcmd = (char *)malloc(cmdlen);  // Get a little more than we need.
  if (bcmd == NULL) return -1;

  strtohex(cmd, bcmd, &cmdlen);

  ret=SCardTransmit(g_card_hdl,
		    o_stdprotocol==SCARD_PROTOCOL_T1 ? 
		    SCARD_PCI_T1 : SCARD_PCI_T0,
		    bcmd, cmdlen, &scir,
		    (BYTE *) outbuff,olen);

  if (ret != 0)
    {
      print_sc_error(ret);
      return ret;
    }
  
  if (*olen==2) {
    switch ((unsigned char)outbuff[0]) {
    case 0x61:
    case 0x9f:
      if (outbuff[1]==0) 
        {
          break;
        }
      g_getresponse[4]=outbuff[1];

      *olen=MAXBUFF;
      ret=SCardTransmit(g_card_hdl,
			o_stdprotocol==SCARD_PROTOCOL_T1 ? SCARD_PCI_T1 : SCARD_PCI_T0,
			g_getresponse,sizeof(g_getresponse),&scir,
			(BYTE *)outbuff,olen);

  if (ret != 0)
    {
      print_sc_error(ret);
      return ret;
    }


    }
  }
  return 0;
}

unsigned char
hinibble(unsigned char c)
{
  unsigned char k;

  k = (c >> 4) & 0x0f;
  if (k == 0x0f)
    return 0;
  else
    return (k + '0');
}

unsigned char
lonibble(unsigned char c)
{
  unsigned char k;

  k = c & 0x0f;
  if (k == 0x0f)
    return 0;
  else
    return (k + '0');
}

int do_gsm(unsigned char *challenge, unsigned char *response, int *resplen, 
	   unsigned char *ckey, int *ckeylen)
{
  unsigned char buf[MAXBUFF], buff2[MAXBUFF], buff3[MAXBUFF];
  int i;
  DWORD len;

  strcpy(buff2, RUN_GSM);
  bzero(&buff3, MAXBUFF);

  for (i = 0; i < 16; i++)
    {
      sprintf(buff3,"%02X",challenge[i]);
      strcat(buff2, buff3);
    }

  //  printf("(in do_gsm) Sending in : %s\n",buff2);
  len = MAXBUFF;
  card_io(buff2, buf, &len);

  /*printf("Response : ");
  
  for (i=0; i<4; i++)
    printf("%02X ",buf[i]);
  */
  memcpy(response, &buf[0], 4);
  /*
  printf("\nCipher Key : ");

  for (i=4; i<12; i++)
    printf("%02X ",buf[i]);
  */
  memcpy(ckey, &buf[4], 8);

  //  printf("\n");
  return 0;
}


int sc_need_init()
{
  return need_init;
}

int init_smartcard(char *pin)
{
  LPSTR mszReaders;
  long ret;
  DWORD size, dwState, dwProtocol, dwAtrLen;
  DWORD len;
  unsigned char *buf; 
  unsigned char buff2[MAXBUFF], buff3[MAXBUFF];
  unsigned char imsi[20];
  int i;
  char *s, *dbuf;
  BYTE pbAtr[MAX_ATR_SIZE];

  buf = (unsigned char *)malloc(MAXBUFF);

  g_sc_context = 0;
  g_card_hdl = 0;
  o_stdprotocol = 0;


  // First, get a context for us to work with the card.
  ret = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &g_sc_context);
  if (ret != SCARD_S_SUCCESS)
    {
      printf("Error!  Couldn't establish Smart Card Context!  (Is pcscd loaded?)\n");
      free(buf);
      buf = NULL;
      return -1;
    }

  // Find out the size of the buffer that we need in order to get our
  // reader information back.
  ret = SCardListReaders(g_sc_context, NULL, NULL, &readerstrlen);
  if (ret != SCARD_S_SUCCESS)
    {
      print_sc_error(ret);
      free(buf);
      buf = NULL;
      return -1;
    }

  o_readername = (char *)malloc(readerstrlen);
  if (o_readername == NULL)
    {
      printf("Couldn't allocate memory for reader info string!\n");
    }

  ret = SCardListReaders(g_sc_context, NULL, o_readername, &readerstrlen);
  if (ret != SCARD_S_SUCCESS)
    {
      printf("Couldn't list smart card readers!\n");
      print_sc_error(ret);
      free(buf);
      buf = NULL;
      return -1;
    }

  // We may have found more than one reader, but we are only going to use 
  // the first one we found.  This could be changed in the future if a
  // situation presents itself that needs more than one reader.  In that
  // case, the string that is returned contains the names of all of the readers
  // with the last name being terminated with a double NULL. \0\0.
  printf("Found reader : %s\n",o_readername);

  while (1)
    {
      ret = SCardConnect(g_sc_context, o_readername,
			 SCARD_SHARE_SHARED,
			 SCARD_PROTOCOL_T0, 
			 &g_card_hdl, &o_stdprotocol);
      
      if (ret == SCARD_S_SUCCESS) break;

      if (ret == SCARD_E_NO_SMARTCARD)
	{
	  printf("Please insert a smart card!\n");
	  sleep(2);
	  continue;
	} else {
	  printf("An unchecked error happened!\n");
	  print_sc_error(ret);
	  break;
	}
    }

  dwState = 0;
  dwProtocol = 0;
  dwAtrLen = 0;
  size = 50;
  mszReaders= (LPSTR) malloc(size);
  bzero(mszReaders, 50);
  bzero(&pbAtr, MAX_ATR_SIZE);
  ret = SCardStatus(g_card_hdl, mszReaders, &size, &dwState, &dwProtocol, pbAtr, &dwAtrLen);
  if (ret != SCARD_S_SUCCESS)
    {
      print_sc_error(ret);
      free(mszReaders);
      free(buf);
      buf = NULL;
      exit(1);                /* Need to fix this! */
    }

  switch (dwState)
    {
    case SCARD_ABSENT:
      printf("There is no card in the reader.\n");
      break;
    case SCARD_PRESENT:
      printf("The card needs to be moved to a position that the reader can use!\n");
      break;
    case SCARD_SWALLOWED:
      printf("Card is ready, but not powered.\n");
      break;
    case SCARD_POWERED:
      printf("Card is powered, but we aren't sure of the mode of the card!\n");
      break;
    }

  /* select the Master File */                
  len=MAXBUFF;
  card_io(SELECT_MF, buf, &len);

  /* select DF_GSM */
  len=MAXBUFF;
  card_io(SELECT_DF_GSM, buf, &len); 

  if (!(buf[13] & 0x80))
    {
      if (pin == NULL) return -1;   /* We don't have a pin, but we need one. */

      strcpy(buff2, "A020000108");
      for (i = 0;i<strlen(pin); i++)
	{
	  sprintf(buff3, "%02X",pin[i]);
	  strcat(buff2, buff3);
	}
      for (i = strlen(pin); i < 8; i++)
	{
	  strcat(buff2, "FF");             /* Pad it to 8 bytes */
	}
      len=MAXBUFF;
      card_io(buff2, buf, &len);
    }

  len=MAXBUFF;
  dbuf = (char *)malloc(MAXBUFF);
  card_io(SELECT_EF_IMSI,dbuf,&len);

   len=MAXBUFF;
   bzero(dbuf,MAXBUFF);
  card_io(GET_IMSI,dbuf,&len);
 
  s = imsi;
  *s++ = hinibble(dbuf[1]);

  for (i = 2; i<9; i++)
    {
      *s++ = lonibble(dbuf[i]);
      *s++ = hinibble(dbuf[i]);
    }
  *s = '\0';

  free(buf);
  buf = NULL;
  free(dbuf);
  dbuf = NULL;

  printf("IMSI = %s\n",imsi);
  need_init = 0;
  return 0;
}


int close_smartcard()
{
  long ret;

  //  printf("Closing smartcard...\n");
  if (g_card_hdl) 
    {
      ret = SCardDisconnect(g_card_hdl, SCARD_UNPOWER_CARD);
      if (ret != SCARD_S_SUCCESS) printf("Couldn't disconnect!\n");
      g_card_hdl = 0;
    }

  if (g_sc_context)
    {
      ret = SCardReleaseContext(g_sc_context);
      if (ret != SCARD_S_SUCCESS) printf("Couldn't release card context!\n");
      g_sc_context = 0;
    }

  return 0;
}


#endif
