%{
/**
 * A client-side 802.1x implementation supporting EAP/TLS
 *
 * This code is released under both the GPL version 2 and BSD licenses.
 * Either license may be used.  The respective licenses are found below.
 *
 * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
 * All Rights Reserved
 *
 * --- GPL Version 2 License ---
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * --- BSD License ---
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the University of
 *       Maryland at College Park and its contributors.
 *  - Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*******************************************************************
 * Grammar for configuration file
 * 
 * File: config_grammar.y
 *
 * Authors: bdpayne@cs.umd.edu, npetroni@cs.umd.edu
 *
 * $Id: config_grammar.y,v 1.10 2003/09/18 23:03:38 chessing Exp $
 * $Date: 2003/09/18 23:03:38 $
 * $Log: config_grammar.y,v $
 * Revision 1.10  2003/09/18 23:03:38  chessing
 * Memory leak fixes, inital implementation of EAP-SIM (draft 5 only).  EAP-SIM still has a lot of bugs to be fixed, and doesn't key correctly yet.  EAP-SIM also requires libpcsc, and pthread.  To enable EAP-SIM use the --enable-eap-sim flag on configure.
 *
 * Revision 1.9  2003/07/01 05:02:43  chessing
 * Code cleanups, added config option that will seperate phase 1, and phase 2 username.  Allows use of "Anonymous" in phase 1 response ID.
 *
 * Revision 1.8  2003/06/06 06:07:34  chessing
 * Applied patches submitted by Brian Fundakowski Feldman from Fujitsu
 *
 * Revision 1.7  2003/05/20 01:44:47  chessing
 * Fixed problems with the -u option.  Added patch for rekeying with Agere based wireless cards.  Cleaned up userconf.c.  And, added first hack at TTLS-CHAP. (Still doesn't work. ;)
 *
 * Revision 1.6  2003/03/28 22:54:00  chessing
 * Patch to fix endianess issues between PPC and x86, with TTLS-PAP.
 *
 * Revision 1.5  2003/03/20 20:14:39  chessing
 * Many patches from Xerox PARC.  They include fixes to show-stoppers, better roaming support, CRLs and chain verification, patch to allow spaces in ESSIDs, some state machine timing issues, and prism2 card fixes.
 *
 * Revision 1.4  2003/03/11 00:13:07  npetroni
 * allow number-only ssids
 *
 * Revision 1.3  2003/03/05 21:18:41  npetroni
 * migration to libdnet. Please notify developers of build problems AFTER doing a cvs update.
 *
 * Revision 1.2  2003/03/04 06:10:08  npetroni
 * changes to allow a program to be run after successful authentication
 *
 * Revision 1.1  2003/03/01 06:55:39  npetroni
 * modified config parse to use lex and yacc for parsing. this may break stuff temporarily, but is good in the long run
 *
 *
 *
 *******************************************************************/  
  
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
  
#include "userconf.h"
  
int yylex(void);  
int yyerror(char *err);
char *config_netname;
char *tmpstring;
int tmpnum;

%}

%union {
        char    *str;
        int     num;
}


%token <num>    TK_ID
%token <num>    TK_CERT
%token <num>    TK_KEY
%token <num>    TK_ROOT
%token <num>    TK_ROOTS
%token <num>    TK_CRLS
%token <num>    TK_AUTH
%token <num>    TK_FIRST_AUTH
%token <num>    TK_AFTER_AUTH
%token <num>    TK_TYPE
%token <num>    TK_PREF
%token <num>    TK_CHUNK_SIZE
%token <num>    TK_RAND_FILE
%token <num>    TK_PASSWORD
%token <num>    TK_PHASE2AUTH
%token <num>    TK_PHASE2ID
%token <str>    TK_NUMBER
%token <str>    TK_STRING
%token <str>    TK_SPC_STRING
%token <str>    TK_COMMAND

%type <str>     netname
%type <str>     filename
%type <str>     command
%type <num>     tagvalpair
%type <num>     idpair
%type <num>     certpair
%type <num>     keypair
%type <num>     rootpair
%type <num>     rootspair
%type <num>     crlspair
%type <num>     authpair
%type <num>     typepair
%type <num>     prefpair
%type <num>     chunkpair
%type <num>     randpair
%type <num>     firstauthpair
%type <num>     afterauthpair
%type <num>     passwordpair
%type <num>     phase2authpair
%type <num>     phase2idpair
%type <str>     namenum

%%
statements		: /* empty */
			| statements statement 
			| error  {  return -1; }
			;

statement               : netname ':' tagvalpair {
               if (strcmp($1, config_netname) == 0) {
                switch($3) {
		case TK_ID:
		  set_username(tmpstring,0) ;
		  break;
		case TK_CERT:
                  set_client_cert(tmpstring);
		  break;
		case TK_KEY:
                  set_key_file(tmpstring);
		  break;
		case TK_ROOT:
		  set_root_cert(tmpstring);
		  break;
		case TK_ROOTS:
		  set_root_dir(tmpstring);
		  break;
		case TK_CRLS:
		  set_crl_dir(tmpstring);
		  break;
		case TK_AUTH:
                  set_auth(tmpstring);
		  break;
		case TK_TYPE:
                  set_client_type(tmpstring);
		  break;
		case TK_PREF:
                  set_preferred_auth(tmpstring);
		  break;
		case TK_CHUNK_SIZE:
                  set_chunk_size(tmpnum);
		  break;
		case TK_RAND_FILE:
                  set_random_file(tmpstring);
		  break;
		case TK_FIRST_AUTH:
                  set_first_auth(tmpstring);
		  break;
		case TK_AFTER_AUTH:
                  set_after_auth(tmpstring);
		  break;
		case TK_PHASE2AUTH:
		  set_phase2auth(tmpstring);
		  break;
		case TK_PHASE2ID:
		  set_phase2id(tmpstring);
		  break;
		case TK_PASSWORD:
                  set_password(tmpstring);
		  break;
		}
	       }
                   }
                        ;

netname                 : TK_NUMBER { $$ = $1; }
                        | TK_STRING { $$ = $1; }
                        | TK_SPC_STRING { $$ = $1; }
                        ;

namenum                 : TK_STRING { $$ = $1; }
                        | TK_NUMBER { $$ = $1; }
                        ;

filename                : TK_STRING { $$ = $1; }
                        | TK_SPC_STRING { $$ = $1; }
                        ;
 
command                 : TK_STRING { $$ = $1; }
                        | TK_SPC_STRING { $$ = $1; }
                        | TK_COMMAND { $$ = $1; }
                        ;
 
tagvalpair              : idpair { $$ = $1; }
                        | certpair {$$ = $1; }
                        | keypair { $$ = $1; }
                        | rootspair { $$ = $1; }
                        | crlspair { $$ = $1; }
                        | rootpair { $$ = $1; }
                        | authpair { $$ = $1; }  
                        | typepair { $$ = $1; }  
                        | prefpair { $$ = $1; }  
                        | chunkpair { $$ = $1; } 
                        | randpair { $$ = $1; }  
                        | firstauthpair { $$ = $1;}
                        | afterauthpair { $$ = $1; }
                        | passwordpair { $$ = $1; }
                        | phase2authpair { $$ = $1; }
                        | phase2idpair { $$ = $1; }
                        ;

idpair                  : TK_ID '=' namenum {
                            tmpstring = $3;
			    $$ = TK_ID;
                          }
                        ;

certpair                : TK_CERT '=' filename {
                            tmpstring = $3;
			    $$ = TK_CERT;
                          }
                        ;

keypair                 : TK_KEY '=' filename {
                            tmpstring = $3;
			    $$ = TK_KEY;
                          }
                        ; 

rootpair                : TK_ROOT '=' filename { 
                            tmpstring = $3;
			    $$ = TK_ROOT;
                          }
                        ; 
rootspair               : TK_ROOTS '=' filename {
                            tmpstring = $3;
			    $$ = TK_ROOTS;
                          }
                        ;

crlspair                : TK_CRLS '=' filename {
                            tmpstring = $3;
			    $$ = TK_CRLS;
                          }
                        ;

authpair                : TK_AUTH '=' TK_STRING {
                            tmpstring = $3;
			    $$ = TK_AUTH;
                          }
                        ;

typepair                : TK_TYPE '=' TK_STRING {
                            tmpstring = $3;
			    $$ = TK_TYPE;
                          }
                        ;

prefpair                : TK_PREF '=' TK_STRING {
                            tmpstring = $3;
			    $$ = TK_PREF;
                          }
                        ;

chunkpair               : TK_CHUNK_SIZE '=' TK_NUMBER {
                            tmpnum = atoi($3);
			    $$ = TK_CHUNK_SIZE;
                          }
                        ;

randpair                : TK_RAND_FILE '=' filename {
                            tmpstring = $3;
			    $$ = TK_RAND_FILE;
                          }
                        ;

firstauthpair           : TK_FIRST_AUTH '=' command {
                            tmpstring = $3;
			    $$ = TK_FIRST_AUTH;
                          }
                        ;

afterauthpair           : TK_AFTER_AUTH '=' command {
                            tmpstring = $3;
			    $$ = TK_AFTER_AUTH;
                          }
                        ;

passwordpair            : TK_PASSWORD '=' namenum {
                            tmpstring = $3;
			    $$ = TK_PASSWORD;
                          }
                        ;

phase2idpair          : TK_PHASE2ID '=' TK_STRING {
                            tmpstring = $3;
			    $$ = TK_PHASE2ID;
                          }
                        ;

phase2authpair          : TK_PHASE2AUTH '=' TK_STRING {
                            tmpstring = $3;
			    $$ = TK_PHASE2AUTH;
                          }
                        ;

%%

