/**
 * A client-side 802.1x implementation supporting EAP/TLS
 *
 * This code is released under both the GPL version 2 and BSD licenses.
 * Either license may be used.  The respective licenses are found below.
 *
 * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
 * All Rights Reserved
 *
 * --- GPL Version 2 License ---
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * --- BSD License ---
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the University of
 *       Maryland at College Park and its contributors.
 *  - Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************
 * The driver function for a Linux application layer EAPOL 
 * implementation
 * File: dot1x_globals.c
 *
 * Authors: Chris.Hessing@utah.edu, Terry.Simons@utah.edu
 *
 * $Id: dot1x_globals.c,v 1.7 2003/06/15 23:16:19 chessing Exp $
 * $Date: 2003/06/15 23:16:19 $
 * $Log: dot1x_globals.c,v $
 * Revision 1.7  2003/06/15 23:16:19  chessing
 * Bug fixes, and GUI Tools additions.
 *
 * Revision 1.6  2003/06/14 20:39:07  chessing
 * Added the following :
 * 	New daemon mode.  Will fork for each real interface, and start a state machine.
 * 	New GUI interface. Still needs work, but has the basics to do a few things.  Common GUI tools source can be found in the gui_tools directory.
 * 	Uses shared memory if available.  If shared memory isn't available, daemon mode is disabled.
 *
 * Revision 1.5  2003/06/11 21:18:19  chessing
 * Cleaned up a lot of memory leaks, added hooks for MEMWATCH to debug.
 *
 * Revision 1.4  2003/03/26 03:39:46  chessing
 * A fix to keyblock handler code when an AP sends back an invalid frame. (Such as D-Link DWL-AP900+)
 *
 * Revision 1.3  2003/03/25 23:59:26  chessing
 * Added support for authentication method specific keyblock generation, and added dynamic WEP support for TTLS.
 *
 * Revision 1.2  2003/03/04 06:10:08  npetroni
 * changes to allow a program to be run after successful authentication
 *
 * Revision 1.1  2003/01/02 19:29:37  chessing
 * Update to bring the xsupplicant code current with the development work.
 *
 *
 *******************************************************************/
#include "dot1x_globals.h"

#if MEMWATCH
#include "memwatch.h"
#endif

struct dot1x_globals one_x_globals;
int is_daemon=0;

void initalize_dot1x_globals() // Set up the dot1x_globals structure.
{
}

void clean_dot1x_globals()     // Free everything, call this on quit.
{
}

void set_daemon(int newval)
{
  is_daemon = newval;
}

int get_daemon()
{
#if NO_DAEMON
  return 0;
#else
  return is_daemon;
#endif
}

int get_authWhile()
{
    return one_x_globals.authWhile;
}

void set_authWhile(int newAuthWhile)
{
    one_x_globals.authWhile = newAuthWhile;
}

int get_aWhile()
{
    return one_x_globals.aWhile;
}

void set_aWhile(int newAWhile)
{
    one_x_globals.aWhile = newAWhile;
}

int get_heldWhile()
{
    return one_x_globals.heldWhile;
}

void set_heldWhile(int newHeldWhile)
{
    one_x_globals.heldWhile = newHeldWhile;
}

int get_quietWhile()
{
    return one_x_globals.quietWhile;
}

void set_quietWhile(int newQuietWhile)
{
    one_x_globals.quietWhile = newQuietWhile;
}

int get_reAuthWhen()
{
    return one_x_globals.reAuthWhen;
}

void set_reAuthWhen(int newReAuthWhen)
{
    one_x_globals.reAuthWhen = newReAuthWhen;
}

int get_startWhen()
{
    return one_x_globals.startWhen;
}

void set_startWhen(int newStartWhen)
{
    one_x_globals.startWhen = newStartWhen;
}

int get_txWhen()
{
    return one_x_globals.txWhen;
}

void set_txWhen(int newTxWhen)
{
    one_x_globals.txWhen = newTxWhen;
}

int get_initialize()
{
    return one_x_globals.initialize;
}

void set_initialize(int newInitialize)
{
    one_x_globals.initialize = newInitialize;
}

int get_suppStatus()
{
    return one_x_globals.suppStatus;
}

void set_suppStatus(int newSuppStatus)
{
    one_x_globals.suppStatus = newSuppStatus;
}

int get_tick()
{
    return one_x_globals.tick;
}

void set_tick(int newTick)
{
    one_x_globals.tick = newTick;
}

int get_userLogoff()
{
    return one_x_globals.userLogoff;
}

void set_userLogoff(int newUserLogoff)
{
    one_x_globals.userLogoff = newUserLogoff;
}

int get_logoffSent()
{
    return one_x_globals.logoffSent;
}

void set_logoffSent(int newLogoffSent)
{
    one_x_globals.logoffSent = newLogoffSent;
}

int get_reqId()
{
    return one_x_globals.reqId;
}

void set_reqId(int newReqId)
{
    one_x_globals.reqId = newReqId;
}

int get_reqAuth()
{
    return one_x_globals.reqAuth;
}

void set_reqAuth(int newReqAuth)
{
    one_x_globals.reqAuth = newReqAuth;
}

int get_eapSuccess()
{
    return one_x_globals.eapSuccess;
}

void set_eapSuccess(int newEapSuccess)
{
    one_x_globals.eapSuccess = newEapSuccess;
}

int get_eapFail()
{
    return one_x_globals.eapFail;
}

void set_eapFail(int newEapFail)
{
    one_x_globals.eapFail = newEapFail;
}

int get_startCount()
{
    return one_x_globals.startCount;
}

void set_startCount(int newStartCount)
{
    one_x_globals.startCount = newStartCount;
}

int get_previousId()
{
    return one_x_globals.previousId;
}

void set_previousId(int newPreviousId)
{
    one_x_globals.previousId = newPreviousId;
}

int get_receivedId()
{
    return one_x_globals.receivedId;
}

void set_receivedId(int newReceivedId)
{
    one_x_globals.receivedId = newReceivedId;
}

int get_rxKey()
{
    return one_x_globals.rxKey;
}

void set_rxKey(int newRxKey)
{
    one_x_globals.rxKey = newRxKey;
}

int get_authType()
{
    return one_x_globals.authType;
}

void set_authType(int newAuthType)
{
    one_x_globals.authType = newAuthType;
}


int get_haveKey()
{
   return one_x_globals.haveKey;
}

void set_haveKey(int newHaveKey)
{
    one_x_globals.haveKey = newHaveKey;
}

int get_authCount()
{
  return one_x_globals.authCount;
}

void reset_authCount()
{
  one_x_globals.authCount = 0;
}

void inc_authCount() 
{
  one_x_globals.authCount++;
}

int get_authMethod()
{
  return one_x_globals.currentAuth;
}

void set_authMethod(int newtype)
{
  one_x_globals.currentAuth = newtype;
}
