/**
 * A client-side 802.1x implementation supporting EAP/TLS
 *
 * This code is released under both the GPL version 2 and BSD licenses.
 * Either license may be used.  The respective licenses are found below.
 *
 * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
 * All Rights Reserved
 *
 * --- GPL Version 2 License ---
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * --- BSD License ---
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the University of
 *       Maryland at College Park and its contributors.
 *  - Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************
 * The driver function for a Linux application layer EAPOL 
 * implementation
 * File: logging.c
 *
 * Authors: Chris.Hessing@utah.edu, Terry.Simons@utah.edu
 *
 * $Id: logging.c,v 1.10 2003/09/21 03:59:35 chessing Exp $
 * $Date: 2003/09/21 03:59:35 $
 * $Log: logging.c,v $
 * Revision 1.10  2003/09/21 03:59:35  chessing
 * Various code cleanups, and other updates.  Smartcard functions for EAP-SIM are more stable.
 *
 * Revision 1.9  2003/06/14 20:45:58  chessing
 * Quick change to show interface information in the log files.
 *
 * Revision 1.8  2003/06/11 21:18:20  chessing
 * Cleaned up a lot of memory leaks, added hooks for MEMWATCH to debug.
 *
 * Revision 1.7  2003/06/08 02:57:19  chessing
 * Added logging to file support, error checking after all mallocs.
 *
 * Revision 1.6  2003/01/09 21:49:51  galimorerpg
 * Logging Updates.
 *
 * Revision 1.5  2003/01/09 21:36:52  galimorerpg
 * Logging Updates.
 *
 * Revision 1.4  2003/01/09 21:20:17  galimorerpg
 * Logging updates.
 *
 * Revision 1.3  2003/01/09 20:48:32  galimorerpg
 * Logging updates.
 *
 * Revision 1.2  2003/01/09 20:17:27  galimorerpg
 * Logging Updates
 *
 * Revision 1.1  2003/01/02 19:29:37  chessing
 * Update to bring the xsupplicant code current with the development work.
 *
 *
 *******************************************************************/

#include "logging.h"
#include <stdarg.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include "frame_handlers/os_frame_funcs.h"

#if MEMWATCH
#include "memwatch.h"
#endif

int debug_level;
int print_to_screen;
FILE *log_handle = NULL;

void init_logfile(char *filename)
{
  debug_level = DEBUG_NORMAL;
  print_to_screen = 0;
  if (filename == NULL)
    {
      print_to_screen = 1;
      return;
    }

  log_handle = fopen(filename, "w");
  if (log_handle == NULL)
    {
      printf("!!! Couldn't create logfile.  Logging will default to the screen.\
n");
      print_to_screen = 1;
    }
}

void xlogf(int level, int force_to_screen, char *fmt, ...)
{
  time_t *cur_time;
  char *datestr;

  if((level <= debug_level) && (fmt != NULL))
    {
      va_list ap;
      va_start(ap, fmt);

      if ((print_to_screen == 0) && (log_handle != NULL))
        {
	  if (force_to_screen != FORCE_NO_LOG)
	    {
	      cur_time = (time_t *)malloc(sizeof(time_t));
	      time(cur_time);
	      datestr = ctime(cur_time);
	      datestr[strlen(datestr)-1] = 0x00;   // Strip off the \n
	      fprintf(log_handle,"%s (%s) - ",datestr,get_dev_name());
	      if (cur_time != NULL) free(cur_time);
	      vfprintf(log_handle, fmt, ap);
	      fflush(log_handle);
	    } 
	  if (force_to_screen == FORCE)
	    {
	      vprintf(fmt, ap);
	    }
        } else {
          vprintf(fmt, ap);
        }

      va_end(ap);
    }
}

